pub trait Float:
Num
+ Copy
+ NumCast
+ PartialOrd
+ Neg<Output = Self> {
Show 60 methods
// Required methods
fn nan() -> Self;
fn infinity() -> Self;
fn neg_infinity() -> Self;
fn neg_zero() -> Self;
fn min_value() -> Self;
fn min_positive_value() -> Self;
fn max_value() -> Self;
fn is_nan(self) -> bool;
fn is_infinite(self) -> bool;
fn is_finite(self) -> bool;
fn is_normal(self) -> bool;
fn classify(self) -> FpCategory;
fn floor(self) -> Self;
fn ceil(self) -> Self;
fn round(self) -> Self;
fn trunc(self) -> Self;
fn fract(self) -> Self;
fn abs(self) -> Self;
fn signum(self) -> Self;
fn is_sign_positive(self) -> bool;
fn is_sign_negative(self) -> bool;
fn mul_add(self, a: Self, b: Self) -> Self;
fn recip(self) -> Self;
fn powi(self, n: i32) -> Self;
fn powf(self, n: Self) -> Self;
fn sqrt(self) -> Self;
fn exp(self) -> Self;
fn exp2(self) -> Self;
fn ln(self) -> Self;
fn log(self, base: Self) -> Self;
fn log2(self) -> Self;
fn log10(self) -> Self;
fn max(self, other: Self) -> Self;
fn min(self, other: Self) -> Self;
fn abs_sub(self, other: Self) -> Self;
fn cbrt(self) -> Self;
fn hypot(self, other: Self) -> Self;
fn sin(self) -> Self;
fn cos(self) -> Self;
fn tan(self) -> Self;
fn asin(self) -> Self;
fn acos(self) -> Self;
fn atan(self) -> Self;
fn atan2(self, other: Self) -> Self;
fn sin_cos(self) -> (Self, Self);
fn exp_m1(self) -> Self;
fn ln_1p(self) -> Self;
fn sinh(self) -> Self;
fn cosh(self) -> Self;
fn tanh(self) -> Self;
fn asinh(self) -> Self;
fn acosh(self) -> Self;
fn atanh(self) -> Self;
fn integer_decode(self) -> (u64, i16, i8);
// Provided methods
fn epsilon() -> Self { ... }
fn is_subnormal(self) -> bool { ... }
fn to_degrees(self) -> Self { ... }
fn to_radians(self) -> Self { ... }
fn clamp(self, min: Self, max: Self) -> Self { ... }
fn copysign(self, sign: Self) -> Self { ... }
}Expand description
Generic trait for floating point numbers
This trait is only available with the std feature, or with the libm feature otherwise.
Required Methods
Sourcefn nan() -> Self
fn nan() -> Self
Returns the NaN value.
use num_traits::Float;
let nan: f32 = Float::nan();
assert!(nan.is_nan());Sourcefn infinity() -> Self
fn infinity() -> Self
Returns the infinite value.
use num_traits::Float;
use std::f32;
let infinity: f32 = Float::infinity();
assert!(infinity.is_infinite());
assert!(!infinity.is_finite());
assert!(infinity > f32::MAX);Sourcefn neg_infinity() -> Self
fn neg_infinity() -> Self
Returns the negative infinite value.
use num_traits::Float;
use std::f32;
let neg_infinity: f32 = Float::neg_infinity();
assert!(neg_infinity.is_infinite());
assert!(!neg_infinity.is_finite());
assert!(neg_infinity < f32::MIN);Sourcefn neg_zero() -> Self
fn neg_zero() -> Self
Returns -0.0.
use num_traits::{Zero, Float};
let inf: f32 = Float::infinity();
let zero: f32 = Zero::zero();
let neg_zero: f32 = Float::neg_zero();
assert_eq!(zero, neg_zero);
assert_eq!(7.0f32/inf, zero);
assert_eq!(zero * 10.0, zero);Sourcefn min_value() -> Self
fn min_value() -> Self
Returns the smallest finite value that this type can represent.
use num_traits::Float;
use std::f64;
let x: f64 = Float::min_value();
assert_eq!(x, f64::MIN);Sourcefn min_positive_value() -> Self
fn min_positive_value() -> Self
Returns the smallest positive, normalized value that this type can represent.
use num_traits::Float;
use std::f64;
let x: f64 = Float::min_positive_value();
assert_eq!(x, f64::MIN_POSITIVE);Sourcefn max_value() -> Self
fn max_value() -> Self
Returns the largest finite value that this type can represent.
use num_traits::Float;
use std::f64;
let x: f64 = Float::max_value();
assert_eq!(x, f64::MAX);Sourcefn is_nan(self) -> bool
fn is_nan(self) -> bool
Returns true if this value is NaN and false otherwise.
use num_traits::Float;
use std::f64;
let nan = f64::NAN;
let f = 7.0;
assert!(nan.is_nan());
assert!(!f.is_nan());Sourcefn is_infinite(self) -> bool
fn is_infinite(self) -> bool
Returns true if this value is positive infinity or negative infinity and
false otherwise.
use num_traits::Float;
use std::f32;
let f = 7.0f32;
let inf: f32 = Float::infinity();
let neg_inf: f32 = Float::neg_infinity();
let nan: f32 = f32::NAN;
assert!(!f.is_infinite());
assert!(!nan.is_infinite());
assert!(inf.is_infinite());
assert!(neg_inf.is_infinite());Sourcefn is_finite(self) -> bool
fn is_finite(self) -> bool
Returns true if this number is neither infinite nor NaN.
use num_traits::Float;
use std::f32;
let f = 7.0f32;
let inf: f32 = Float::infinity();
let neg_inf: f32 = Float::neg_infinity();
let nan: f32 = f32::NAN;
assert!(f.is_finite());
assert!(!nan.is_finite());
assert!(!inf.is_finite());
assert!(!neg_inf.is_finite());Sourcefn is_normal(self) -> bool
fn is_normal(self) -> bool
Returns true if the number is neither zero, infinite,
subnormal, or NaN.
use num_traits::Float;
use std::f32;
let min = f32::MIN_POSITIVE; // 1.17549435e-38f32
let max = f32::MAX;
let lower_than_min = 1.0e-40_f32;
let zero = 0.0f32;
assert!(min.is_normal());
assert!(max.is_normal());
assert!(!zero.is_normal());
assert!(!f32::NAN.is_normal());
assert!(!f32::INFINITY.is_normal());
// Values between `0` and `min` are Subnormal.
assert!(!lower_than_min.is_normal());Sourcefn classify(self) -> FpCategory
fn classify(self) -> FpCategory
Returns the floating point category of the number. If only one property is going to be tested, it is generally faster to use the specific predicate instead.
use num_traits::Float;
use std::num::FpCategory;
use std::f32;
let num = 12.4f32;
let inf = f32::INFINITY;
assert_eq!(num.classify(), FpCategory::Normal);
assert_eq!(inf.classify(), FpCategory::Infinite);Sourcefn floor(self) -> Self
fn floor(self) -> Self
Returns the largest integer less than or equal to a number.
use num_traits::Float;
let f = 3.99;
let g = 3.0;
assert_eq!(f.floor(), 3.0);
assert_eq!(g.floor(), 3.0);Sourcefn ceil(self) -> Self
fn ceil(self) -> Self
Returns the smallest integer greater than or equal to a number.
use num_traits::Float;
let f = 3.01;
let g = 4.0;
assert_eq!(f.ceil(), 4.0);
assert_eq!(g.ceil(), 4.0);Sourcefn round(self) -> Self
fn round(self) -> Self
Returns the nearest integer to a number. Round half-way cases away from
0.0.
use num_traits::Float;
let f = 3.3;
let g = -3.3;
assert_eq!(f.round(), 3.0);
assert_eq!(g.round(), -3.0);Sourcefn trunc(self) -> Self
fn trunc(self) -> Self
Return the integer part of a number.
use num_traits::Float;
let f = 3.3;
let g = -3.7;
assert_eq!(f.trunc(), 3.0);
assert_eq!(g.trunc(), -3.0);Sourcefn fract(self) -> Self
fn fract(self) -> Self
Returns the fractional part of a number.
use num_traits::Float;
let x = 3.5;
let y = -3.5;
let abs_difference_x = (x.fract() - 0.5).abs();
let abs_difference_y = (y.fract() - (-0.5)).abs();
assert!(abs_difference_x < 1e-10);
assert!(abs_difference_y < 1e-10);Sourcefn abs(self) -> Self
fn abs(self) -> Self
Computes the absolute value of self. Returns Float::nan() if the
number is Float::nan().
use num_traits::Float;
use std::f64;
let x = 3.5;
let y = -3.5;
let abs_difference_x = (x.abs() - x).abs();
let abs_difference_y = (y.abs() - (-y)).abs();
assert!(abs_difference_x < 1e-10);
assert!(abs_difference_y < 1e-10);
assert!(f64::NAN.abs().is_nan());Sourcefn signum(self) -> Self
fn signum(self) -> Self
Returns a number that represents the sign of self.
1.0if the number is positive,+0.0orFloat::infinity()-1.0if the number is negative,-0.0orFloat::neg_infinity()Float::nan()if the number isFloat::nan()
use num_traits::Float;
use std::f64;
let f = 3.5;
assert_eq!(f.signum(), 1.0);
assert_eq!(f64::NEG_INFINITY.signum(), -1.0);
assert!(f64::NAN.signum().is_nan());Sourcefn is_sign_positive(self) -> bool
fn is_sign_positive(self) -> bool
Returns true if self is positive, including +0.0,
Float::infinity(), and Float::nan().
use num_traits::Float;
use std::f64;
let nan: f64 = f64::NAN;
let neg_nan: f64 = -f64::NAN;
let f = 7.0;
let g = -7.0;
assert!(f.is_sign_positive());
assert!(!g.is_sign_positive());
assert!(nan.is_sign_positive());
assert!(!neg_nan.is_sign_positive());Sourcefn is_sign_negative(self) -> bool
fn is_sign_negative(self) -> bool
Returns true if self is negative, including -0.0,
Float::neg_infinity(), and -Float::nan().
use num_traits::Float;
use std::f64;
let nan: f64 = f64::NAN;
let neg_nan: f64 = -f64::NAN;
let f = 7.0;
let g = -7.0;
assert!(!f.is_sign_negative());
assert!(g.is_sign_negative());
assert!(!nan.is_sign_negative());
assert!(neg_nan.is_sign_negative());Sourcefn mul_add(self, a: Self, b: Self) -> Self
fn mul_add(self, a: Self, b: Self) -> Self
Fused multiply-add. Computes (self * a) + b with only one rounding
error, yielding a more accurate result than an unfused multiply-add.
Using mul_add can be more performant than an unfused multiply-add if
the target architecture has a dedicated fma CPU instruction.
use num_traits::Float;
let m = 10.0;
let x = 4.0;
let b = 60.0;
// 100.0
let abs_difference = (m.mul_add(x, b) - (m*x + b)).abs();
assert!(abs_difference < 1e-10);Sourcefn recip(self) -> Self
fn recip(self) -> Self
Take the reciprocal (inverse) of a number, 1/x.
use num_traits::Float;
let x = 2.0;
let abs_difference = (x.recip() - (1.0/x)).abs();
assert!(abs_difference < 1e-10);Sourcefn powi(self, n: i32) -> Self
fn powi(self, n: i32) -> Self
Raise a number to an integer power.
Using this function is generally faster than using powf
use num_traits::Float;
let x = 2.0;
let abs_difference = (x.powi(2) - x*x).abs();
assert!(abs_difference < 1e-10);Sourcefn powf(self, n: Self) -> Self
fn powf(self, n: Self) -> Self
Raise a number to a floating point power.
use num_traits::Float;
let x = 2.0;
let abs_difference = (x.powf(2.0) - x*x).abs();
assert!(abs_difference < 1e-10);Sourcefn sqrt(self) -> Self
fn sqrt(self) -> Self
Take the square root of a number.
Returns NaN if self is a negative number.
use num_traits::Float;
let positive = 4.0;
let negative = -4.0;
let abs_difference = (positive.sqrt() - 2.0).abs();
assert!(abs_difference < 1e-10);
assert!(negative.sqrt().is_nan());Sourcefn exp(self) -> Self
fn exp(self) -> Self
Returns e^(self), (the exponential function).
use num_traits::Float;
let one = 1.0;
// e^1
let e = one.exp();
// ln(e) - 1 == 0
let abs_difference = (e.ln() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn exp2(self) -> Self
fn exp2(self) -> Self
Returns 2^(self).
use num_traits::Float;
let f = 2.0;
// 2^2 - 4 == 0
let abs_difference = (f.exp2() - 4.0).abs();
assert!(abs_difference < 1e-10);Sourcefn ln(self) -> Self
fn ln(self) -> Self
Returns the natural logarithm of the number.
use num_traits::Float;
let one = 1.0;
// e^1
let e = one.exp();
// ln(e) - 1 == 0
let abs_difference = (e.ln() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn log(self, base: Self) -> Self
fn log(self, base: Self) -> Self
Returns the logarithm of the number with respect to an arbitrary base.
use num_traits::Float;
let ten = 10.0;
let two = 2.0;
// log10(10) - 1 == 0
let abs_difference_10 = (ten.log(10.0) - 1.0).abs();
// log2(2) - 1 == 0
let abs_difference_2 = (two.log(2.0) - 1.0).abs();
assert!(abs_difference_10 < 1e-10);
assert!(abs_difference_2 < 1e-10);Sourcefn log2(self) -> Self
fn log2(self) -> Self
Returns the base 2 logarithm of the number.
use num_traits::Float;
let two = 2.0;
// log2(2) - 1 == 0
let abs_difference = (two.log2() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn log10(self) -> Self
fn log10(self) -> Self
Returns the base 10 logarithm of the number.
use num_traits::Float;
let ten = 10.0;
// log10(10) - 1 == 0
let abs_difference = (ten.log10() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn max(self, other: Self) -> Self
fn max(self, other: Self) -> Self
Returns the maximum of the two numbers.
use num_traits::Float;
let x = 1.0;
let y = 2.0;
assert_eq!(x.max(y), y);Sourcefn min(self, other: Self) -> Self
fn min(self, other: Self) -> Self
Returns the minimum of the two numbers.
use num_traits::Float;
let x = 1.0;
let y = 2.0;
assert_eq!(x.min(y), x);Sourcefn abs_sub(self, other: Self) -> Self
fn abs_sub(self, other: Self) -> Self
The positive difference of two numbers.
- If
self <= other:0:0 - Else:
self - other
use num_traits::Float;
let x = 3.0;
let y = -3.0;
let abs_difference_x = (x.abs_sub(1.0) - 2.0).abs();
let abs_difference_y = (y.abs_sub(1.0) - 0.0).abs();
assert!(abs_difference_x < 1e-10);
assert!(abs_difference_y < 1e-10);Sourcefn cbrt(self) -> Self
fn cbrt(self) -> Self
Take the cubic root of a number.
use num_traits::Float;
let x = 8.0;
// x^(1/3) - 2 == 0
let abs_difference = (x.cbrt() - 2.0).abs();
assert!(abs_difference < 1e-10);Sourcefn hypot(self, other: Self) -> Self
fn hypot(self, other: Self) -> Self
Calculate the length of the hypotenuse of a right-angle triangle given
legs of length x and y.
use num_traits::Float;
let x = 2.0;
let y = 3.0;
// sqrt(x^2 + y^2)
let abs_difference = (x.hypot(y) - (x.powi(2) + y.powi(2)).sqrt()).abs();
assert!(abs_difference < 1e-10);Sourcefn sin(self) -> Self
fn sin(self) -> Self
Computes the sine of a number (in radians).
use num_traits::Float;
use std::f64;
let x = f64::consts::PI/2.0;
let abs_difference = (x.sin() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn cos(self) -> Self
fn cos(self) -> Self
Computes the cosine of a number (in radians).
use num_traits::Float;
use std::f64;
let x = 2.0*f64::consts::PI;
let abs_difference = (x.cos() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn tan(self) -> Self
fn tan(self) -> Self
Computes the tangent of a number (in radians).
use num_traits::Float;
use std::f64;
let x = f64::consts::PI/4.0;
let abs_difference = (x.tan() - 1.0).abs();
assert!(abs_difference < 1e-14);Sourcefn asin(self) -> Self
fn asin(self) -> Self
Computes the arcsine of a number. Return value is in radians in the range [-pi/2, pi/2] or NaN if the number is outside the range [-1, 1].
use num_traits::Float;
use std::f64;
let f = f64::consts::PI / 2.0;
// asin(sin(pi/2))
let abs_difference = (f.sin().asin() - f64::consts::PI / 2.0).abs();
assert!(abs_difference < 1e-10);Sourcefn acos(self) -> Self
fn acos(self) -> Self
Computes the arccosine of a number. Return value is in radians in the range [0, pi] or NaN if the number is outside the range [-1, 1].
use num_traits::Float;
use std::f64;
let f = f64::consts::PI / 4.0;
// acos(cos(pi/4))
let abs_difference = (f.cos().acos() - f64::consts::PI / 4.0).abs();
assert!(abs_difference < 1e-10);Sourcefn atan(self) -> Self
fn atan(self) -> Self
Computes the arctangent of a number. Return value is in radians in the range [-pi/2, pi/2];
use num_traits::Float;
let f = 1.0;
// atan(tan(1))
let abs_difference = (f.tan().atan() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn atan2(self, other: Self) -> Self
fn atan2(self, other: Self) -> Self
Computes the four quadrant arctangent of self (y) and other (x).
x = 0,y = 0:0x >= 0:arctan(y/x)->[-pi/2, pi/2]y >= 0:arctan(y/x) + pi->(pi/2, pi]y < 0:arctan(y/x) - pi->(-pi, -pi/2)
use num_traits::Float;
use std::f64;
let pi = f64::consts::PI;
// All angles from horizontal right (+x)
// 45 deg counter-clockwise
let x1 = 3.0;
let y1 = -3.0;
// 135 deg clockwise
let x2 = -3.0;
let y2 = 3.0;
let abs_difference_1 = (y1.atan2(x1) - (-pi/4.0)).abs();
let abs_difference_2 = (y2.atan2(x2) - 3.0*pi/4.0).abs();
assert!(abs_difference_1 < 1e-10);
assert!(abs_difference_2 < 1e-10);Sourcefn sin_cos(self) -> (Self, Self)
fn sin_cos(self) -> (Self, Self)
Simultaneously computes the sine and cosine of the number, x. Returns
(sin(x), cos(x)).
use num_traits::Float;
use std::f64;
let x = f64::consts::PI/4.0;
let f = x.sin_cos();
let abs_difference_0 = (f.0 - x.sin()).abs();
let abs_difference_1 = (f.1 - x.cos()).abs();
assert!(abs_difference_0 < 1e-10);
assert!(abs_difference_0 < 1e-10);Sourcefn exp_m1(self) -> Self
fn exp_m1(self) -> Self
Returns e^(self) - 1 in a way that is accurate even if the
number is close to zero.
use num_traits::Float;
let x = 7.0;
// e^(ln(7)) - 1
let abs_difference = (x.ln().exp_m1() - 6.0).abs();
assert!(abs_difference < 1e-10);Sourcefn ln_1p(self) -> Self
fn ln_1p(self) -> Self
Returns ln(1+n) (natural logarithm) more accurately than if
the operations were performed separately.
use num_traits::Float;
use std::f64;
let x = f64::consts::E - 1.0;
// ln(1 + (e - 1)) == ln(e) == 1
let abs_difference = (x.ln_1p() - 1.0).abs();
assert!(abs_difference < 1e-10);Sourcefn sinh(self) -> Self
fn sinh(self) -> Self
Hyperbolic sine function.
use num_traits::Float;
use std::f64;
let e = f64::consts::E;
let x = 1.0;
let f = x.sinh();
// Solving sinh() at 1 gives `(e^2-1)/(2e)`
let g = (e*e - 1.0)/(2.0*e);
let abs_difference = (f - g).abs();
assert!(abs_difference < 1e-10);Sourcefn cosh(self) -> Self
fn cosh(self) -> Self
Hyperbolic cosine function.
use num_traits::Float;
use std::f64;
let e = f64::consts::E;
let x = 1.0;
let f = x.cosh();
// Solving cosh() at 1 gives this result
let g = (e*e + 1.0)/(2.0*e);
let abs_difference = (f - g).abs();
// Same result
assert!(abs_difference < 1.0e-10);Sourcefn tanh(self) -> Self
fn tanh(self) -> Self
Hyperbolic tangent function.
use num_traits::Float;
use std::f64;
let e = f64::consts::E;
let x = 1.0;
let f = x.tanh();
// Solving tanh() at 1 gives `(1 - e^(-2))/(1 + e^(-2))`
let g = (1.0 - e.powi(-2))/(1.0 + e.powi(-2));
let abs_difference = (f - g).abs();
assert!(abs_difference < 1.0e-10);Sourcefn asinh(self) -> Self
fn asinh(self) -> Self
Inverse hyperbolic sine function.
use num_traits::Float;
let x = 1.0;
let f = x.sinh().asinh();
let abs_difference = (f - x).abs();
assert!(abs_difference < 1.0e-10);Sourcefn acosh(self) -> Self
fn acosh(self) -> Self
Inverse hyperbolic cosine function.
use num_traits::Float;
let x = 1.0;
let f = x.cosh().acosh();
let abs_difference = (f - x).abs();
assert!(abs_difference < 1.0e-10);Sourcefn atanh(self) -> Self
fn atanh(self) -> Self
Inverse hyperbolic tangent function.
use num_traits::Float;
use std::f64;
let e = f64::consts::E;
let f = e.tanh().atanh();
let abs_difference = (f - e).abs();
assert!(abs_difference < 1.0e-10);Sourcefn integer_decode(self) -> (u64, i16, i8)
fn integer_decode(self) -> (u64, i16, i8)
Returns the mantissa, base 2 exponent, and sign as integers, respectively.
The original number can be recovered by sign * mantissa * 2 ^ exponent.
use num_traits::Float;
let num = 2.0f32;
// (8388608, -22, 1)
let (mantissa, exponent, sign) = Float::integer_decode(num);
let sign_f = sign as f32;
let mantissa_f = mantissa as f32;
let exponent_f = num.powf(exponent as f32);
// 1 * 8388608 * 2^(-22) == 2
let abs_difference = (sign_f * mantissa_f * exponent_f - num).abs();
assert!(abs_difference < 1e-10);Provided Methods
Sourcefn epsilon() -> Self
fn epsilon() -> Self
Returns epsilon, a small positive value.
use num_traits::Float;
use std::f64;
let x: f64 = Float::epsilon();
assert_eq!(x, f64::EPSILON);Panics
The default implementation will panic if f32::EPSILON cannot
be cast to Self.
Sourcefn is_subnormal(self) -> bool
fn is_subnormal(self) -> bool
Returns true if the number is subnormal.
use num_traits::Float;
use std::f64;
let min = f64::MIN_POSITIVE; // 2.2250738585072014e-308_f64
let max = f64::MAX;
let lower_than_min = 1.0e-308_f64;
let zero = 0.0_f64;
assert!(!min.is_subnormal());
assert!(!max.is_subnormal());
assert!(!zero.is_subnormal());
assert!(!f64::NAN.is_subnormal());
assert!(!f64::INFINITY.is_subnormal());
// Values between `0` and `min` are Subnormal.
assert!(lower_than_min.is_subnormal());Sourcefn to_degrees(self) -> Self
fn to_degrees(self) -> Self
Converts radians to degrees.
use std::f64::consts;
let angle = consts::PI;
let abs_difference = (angle.to_degrees() - 180.0).abs();
assert!(abs_difference < 1e-10);Sourcefn to_radians(self) -> Self
fn to_radians(self) -> Self
Converts degrees to radians.
use std::f64::consts;
let angle = 180.0_f64;
let abs_difference = (angle.to_radians() - consts::PI).abs();
assert!(abs_difference < 1e-10);Sourcefn clamp(self, min: Self, max: Self) -> Self
fn clamp(self, min: Self, max: Self) -> Self
Clamps a value between a min and max.
Panics in debug mode if !(min <= max).
use num_traits::Float;
let x = 1.0;
let y = 2.0;
let z = 3.0;
assert_eq!(x.clamp(y, z), 2.0);Sourcefn copysign(self, sign: Self) -> Self
fn copysign(self, sign: Self) -> Self
Returns a number composed of the magnitude of self and the sign of
sign.
Equal to self if the sign of self and sign are the same, otherwise
equal to -self. If self is a NAN, then a NAN with the sign of
sign is returned.
Examples
use num_traits::Float;
let f = 3.5_f32;
assert_eq!(f.copysign(0.42), 3.5_f32);
assert_eq!(f.copysign(-0.42), -3.5_f32);
assert_eq!((-f).copysign(0.42), 3.5_f32);
assert_eq!((-f).copysign(-0.42), -3.5_f32);
assert!(f32::nan().copysign(1.0).is_nan());Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.
Implementations on Foreign Types
Sourceimpl Float for f32
Available on non-crate feature std and crate feature libm only.
impl Float for f32
std and crate feature libm only.fn nan() -> Self
fn infinity() -> Self
fn neg_infinity() -> Self
fn neg_zero() -> Self
fn min_value() -> Self
fn min_positive_value() -> Self
fn epsilon() -> Self
fn max_value() -> Self
fn integer_decode(self) -> (u64, i16, i8)
fn fract(self) -> Self
fn log(self, base: Self) -> Self
fn is_nan(self) -> bool
fn is_infinite(self) -> bool
fn is_finite(self) -> bool
fn is_normal(self) -> bool
fn is_subnormal(self) -> bool
fn clamp(self, min: Self, max: Self) -> Self
fn classify(self) -> FpCategory
fn is_sign_positive(self) -> bool
fn is_sign_negative(self) -> bool
fn min(self, other: Self) -> Self
fn max(self, other: Self) -> Self
fn recip(self) -> Self
fn to_degrees(self) -> Self
fn to_radians(self) -> Self
fn signum(self) -> Self
fn powi(self, n: i32) -> Self
fn abs_sub(self, other: Self) -> Self
fn floor(self) -> Self
fn ceil(self) -> Self
fn round(self) -> Self
fn trunc(self) -> Self
fn abs(self) -> Self
fn mul_add(self, a: Self, b: Self) -> Self
fn powf(self, n: Self) -> Self
fn sqrt(self) -> Self
fn exp(self) -> Self
fn exp2(self) -> Self
fn ln(self) -> Self
fn log2(self) -> Self
fn log10(self) -> Self
fn cbrt(self) -> Self
fn hypot(self, other: Self) -> Self
fn sin(self) -> Self
fn cos(self) -> Self
fn tan(self) -> Self
fn asin(self) -> Self
fn acos(self) -> Self
fn atan(self) -> Self
fn atan2(self, other: Self) -> Self
fn sin_cos(self) -> (Self, Self)
fn exp_m1(self) -> Self
fn ln_1p(self) -> Self
fn sinh(self) -> Self
fn cosh(self) -> Self
fn tanh(self) -> Self
fn asinh(self) -> Self
fn acosh(self) -> Self
fn atanh(self) -> Self
fn copysign(self, other: Self) -> Self
Sourceimpl Float for f64
Available on non-crate feature std and crate feature libm only.
impl Float for f64
std and crate feature libm only.